@@ -77,7 +77,7 @@ func TestParse(t *testing.T) {
 		{"debugger", "Stmt(debugger)"},
 		{"label: var a", "Stmt(label : Decl(var Binding(a)))"},
 		{"yield: var a", "Stmt(yield : Decl(var Binding(a)))"},
-		{"await: var a", "Stmt(await : Decl(var Binding(a)))"},
+		{"function a(){ await: var a }", "Decl(function a Params() Stmt({ Stmt(await : Decl(var Binding(a))) }))"},
 		{"switch (5) {}", "Stmt(switch 5)"},
 		{"switch (5) { case 3: {} default: {}}", "Stmt(switch 5 Clause(case 3 Stmt({ })) Clause(default Stmt({ })))"},
 		{"function a(b) {}", "Decl(function a Params(Binding(b)) Stmt({ }))"},
@@ -116,17 +116,20 @@ func TestParse(t *testing.T) {
 		{"`tmpl`", "Stmt(`tmpl`)"},
 		{"`tmpl${x}`", "Stmt(`tmpl${x}`)"},
 		{"`tmpl${x}tmpl${x}`", "Stmt(`tmpl${x}tmpl${x}`)"},
-		{"import \"pkg\";", "Stmt(import \"pkg\")"},
-		{"import yield from \"pkg\"", "Stmt(import yield from \"pkg\")"},
-		{"import * as yield from \"pkg\"", "Stmt(import * as yield from \"pkg\")"},
-		{"import {yield, for as yield,} from \"pkg\"", "Stmt(import { yield , for as yield , } from \"pkg\")"},
-		{"import yield, * as yield from \"pkg\"", "Stmt(import yield , * as yield from \"pkg\")"},
-		{"import yield, {yield} from \"pkg\"", "Stmt(import yield , { yield } from \"pkg\")"},
-		{"import {yield,} from \"pkg\"", "Stmt(import { yield , } from \"pkg\")"},
-		{"export * from \"pkg\";", "Stmt(export * from \"pkg\")"},
-		{"export * as for from \"pkg\"", "Stmt(export * as for from \"pkg\")"},
-		{"export {if, for as switch} from \"pkg\"", "Stmt(export { if , for as switch } from \"pkg\")"},
+		{`import "pkg";`, `Stmt(import "pkg")`},
+		{`import yield from "pkg"`, `Stmt(import yield from "pkg")`},
+		{`import * as yield from "pkg"`, `Stmt(import * as yield from "pkg")`},
+		{`import {yield, for as yield,} from "pkg"`, `Stmt(import { yield , for as yield , } from "pkg")`},
+		{`import yield, * as yield from "pkg"`, `Stmt(import yield , * as yield from "pkg")`},
+		{`import yield, {yield} from "pkg"`, `Stmt(import yield , { yield } from "pkg")`},
+		{`import {yield,} from "pkg"`, `Stmt(import { yield , } from "pkg")`},
+		{`import {"abc'def" as a} from "pkg"`, `Stmt(import { "abc'def" as a } from "pkg")`},
+		{`export * from "pkg";`, `Stmt(export * from "pkg")`},
+		{`export * as for from "pkg"`, `Stmt(export * as for from "pkg")`},
+		{`export * as "abc'def" from "pkg"`, `Stmt(export * as "abc'def" from "pkg")`},
+		{`export {if, for as switch} from "pkg"`, `Stmt(export { if , for as switch } from "pkg")`},
 		{"export {if, for as switch,}", "Stmt(export { if , for as switch , })"},
+		{`export {"abc'def", "a" as 'b'}`, `Stmt(export { "abc'def" , "a" as 'b' })`},
 		{"export var a", "Stmt(export Decl(var Binding(a)))"},
 		{"export function a(b){}", "Stmt(export Decl(function a Params(Binding(b)) Stmt({ })))"},
 		{"export async function a(b){}", "Stmt(export Decl(async function a Params(Binding(b)) Stmt({ })))"},
@@ -141,7 +144,7 @@ func TestParse(t *testing.T) {
 		{"yield\na = 5", "Stmt(yield) Stmt(a=5)"},
 		{"yield * yield * a", "Stmt((yield*yield)*a)"},
 		{"(yield) => 5", "Stmt(Params(Binding(yield)) => Stmt({ Stmt(return 5) }))"},
-		{"(await) => 5", "Stmt(Params(Binding(await)) => Stmt({ Stmt(return 5) }))"},
+		{"function a(){ (await) => 5 }", "Decl(function a Params() Stmt({ Stmt(Params(Binding(await)) => Stmt({ Stmt(return 5) })) }))"},
 		{"async", "Stmt(async)"},
 		{"async = a", "Stmt(async=a)"},
 		{"async\n= a", "Stmt(async=a)"},
@@ -150,7 +153,7 @@ func TestParse(t *testing.T) {
 		{"async(a)", "Stmt(async(a))"},
 		{"async(a=6, ...b)", "Stmt(async((a=6), ...b))"},
 		{"async(function(){})", "Stmt(async(Decl(function Params() Stmt({ }))))"},
-		{"async\nawait => b", "Stmt(async) Stmt(Params(Binding(await)) => Stmt({ Stmt(return b) }))"},
+		{"function a(){ async\nawait => b }", "Decl(function a Params() Stmt({ Stmt(async) Stmt(Params(Binding(await)) => Stmt({ Stmt(return b) })) }))"},
 		{"a + async\nb", "Stmt(a+async) Stmt(b)"},
 		{"a + async\nfunction f(){}", "Stmt(a+async) Decl(function f Params() Stmt({ }))"},
 		{"class a extends async {}", "Decl(class a extends async)"},
@@ -164,7 +167,7 @@ func TestParse(t *testing.T) {
 		{"function*a(){ x = function b(){ x = yield } }", "Decl(function* a Params() Stmt({ Stmt(x=Decl(function b Params() Stmt({ Stmt(x=yield) }))) }))"},
 		{"function*a(){ (yield) }", "Decl(function* a Params() Stmt({ Stmt((yield)) }))"},
 		{"function*a(){ (yield a) }", "Decl(function* a Params() Stmt({ Stmt((yield a)) }))"},
-		{"let\nawait", "Decl(let Binding(await))"},
+		{"function a(){ let\nawait }", "Decl(function a Params() Stmt({ Decl(let Binding(await)) }))"},
 		{"x = {await}", "Stmt(x={await})"},
 		{"async function a(){ x = {await: 5} }", "Decl(async function a Params() Stmt({ Stmt(x={await: 5}) }))"},
 		{"async function a(){ x = await a }", "Decl(async function a Params() Stmt({ Stmt(x=(await a)) }))"},
@@ -190,7 +193,8 @@ func TestParse(t *testing.T) {
 		{"let [name = 5,, ...{yield}] = z", "Decl(let Binding([ Binding(name = 5), Binding(), ...Binding({ Binding(yield) }) ] = z))"},
 		{"let {} = z", "Decl(let Binding({ } = z))"},
 		{"let {name = 5} = z", "Decl(let Binding({ Binding(name = 5) } = z))"},
-		{"let {await = 5} = z", "Decl(let Binding({ Binding(await = 5) } = z))"},
+		{"function a(){ let {await = 5} = z }", "Decl(function a Params() Stmt({ Decl(let Binding({ Binding(await = 5) } = z)) }))"},
+		{"let {await: a} = z", "Decl(let Binding({ await: Binding(a) } = z))"},
 		{"let {if: name} = z", "Decl(let Binding({ if: Binding(name) } = z))"},
 		{"let {\"string\": name} = z", "Decl(let Binding({ string: Binding(name) } = z))"},
 		{"let {[a = 5]: name} = z", "Decl(let Binding({ [a=5]: Binding(name) } = z))"},
@@ -253,12 +257,14 @@ func TestParse(t *testing.T) {
 		{"x = async a => a++", "Stmt(x=(async Params(Binding(a)) => Stmt({ Stmt(return (a++)) })))"},
 		{"x = async a => {a++}", "Stmt(x=(async Params(Binding(a)) => Stmt({ Stmt(a++) })))"},
 		{"x = async a => await b", "Stmt(x=(async Params(Binding(a)) => Stmt({ Stmt(return (await b)) })))"},
-		{"x = await => a++", "Stmt(x=(Params(Binding(await)) => Stmt({ Stmt(return (a++)) })))"},
+		{"function a(){ x = await => a++ }", "Decl(function a Params() Stmt({ Stmt(x=(Params(Binding(await)) => Stmt({ Stmt(return (a++)) }))) }))"},
 		{"x = a??b", "Stmt(x=(a??b))"},
 		{"x = a[b]", "Stmt(x=(a[b]))"},
+		{"x = a.#b", "Stmt(x=(a.#b))"},
 		{"x = a?.b?.c.d", "Stmt(x=(((a?.b)?.c).d))"},
 		{"x = a?.[b]?.`tpl`", "Stmt(x=((a?.[b])?.`tpl`))"},
 		{"x = a?.(b)", "Stmt(x=(a?.(b)))"},
+		{"x = a?.#b", "Stmt(x=(a?.#b))"},
 		{"x = super(a)", "Stmt(x=(super(a)))"},
 		{"x = a(a,b,...c,)", "Stmt(x=(a(a, b, ...c)))"},
 		{"x = a(...a,...b)", "Stmt(x=(a(...a, ...b)))"},
@@ -266,6 +272,7 @@ func TestParse(t *testing.T) {
 		{"x = new a()", "Stmt(x=(new a))"},
 		{"x = new a(b)", "Stmt(x=(new a(b)))"},
 		{"x = new a().b(c)", "Stmt(x=(((new a).b)(c)))"},
+		{"x = new a().#b", "Stmt(x=((new a).#b))"},
 		{"x = new new.target", "Stmt(x=(new (new.target)))"},
 		{"x = new import.meta", "Stmt(x=(new (import.meta)))"},
 		{"x = import(a)", "Stmt(x=(import(a)))"},
@@ -339,7 +346,7 @@ func TestParse(t *testing.T) {
 		{"a&b&c", "Stmt((a&b)&c)"},
 		{"a|b|c", "Stmt((a|b)|c)"},
 		{"a^b^c", "Stmt((a^b)^c)"},
-		{"a,b,c", "Stmt((a,b),c)"},
+		{"a,b,c", "Stmt(a,b,c)"},
 
 		// regular expressions
 		{"/abc/", "Stmt(/abc/)"},
@@ -391,7 +398,7 @@ func TestParse(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.js, func(t *testing.T) {
-			ast, err := Parse(parse.NewInputString(tt.js))
+			ast, err := Parse(parse.NewInputString(tt.js), Options{})
 			if err != io.EOF {
 				test.Error(t, err)
 			}
@@ -436,7 +443,8 @@ func TestParseError(t *testing.T) {
 		{"for(a;a;a", "expected ) instead of EOF in for statement"},
 		{"for(var [a],b;", "unexpected ; in for statement"},
 		{"for(var [a]=5,{b};", "expected = instead of ; in var statement"},
-		{"for await", "expected ( instead of await in for statement"},
+		{"for await", "expected ( instead of EOF in for statement"},
+		{"function a(){for await", "expected ( instead of await in for statement"},
 		{"async function a(){ for await(a;", "expected of instead of ; in for statement"},
 		{"async function a(){ for await(a in", "expected of instead of in in for statement"},
 		{"for(var a of b", "expected ) instead of EOF in for statement"},
@@ -480,6 +488,7 @@ func TestParseError(t *testing.T) {
 		{"var {...a", "expected } instead of EOF in object binding pattern"},
 		{"var {a}", "expected = instead of EOF in var statement"},
 		{"var 0", "unexpected 0 in binding"},
+		{"const a", "expected = instead of EOF in const statement"},
 		{"x={", "expected } instead of EOF in object literal"},
 		{"x={[a", "expected ] instead of EOF in object literal"},
 		{"x={[a]", "expected : or ( instead of EOF in object literal"},
@@ -529,12 +538,12 @@ func TestParseError(t *testing.T) {
 		{"import yield from", "expected String instead of EOF in import statement"},
 		{"export", "expected *, {, var, let, const, function, async, class, or default instead of EOF in export statement"},
 		{"export *", "expected from instead of EOF in export statement"},
-		{"export * as", "expected Identifier instead of EOF in export statement"},
+		{"export * as", "expected Identifier or String instead of EOF in export statement"},
 		{"export * as if", "expected from instead of EOF in export statement"},
 		{"export {", "expected } instead of EOF in export statement"},
 		{"export {yield", "expected } instead of EOF in export statement"},
 		{"export {yield,", "expected } instead of EOF in export statement"},
-		{"export {yield as", "expected Identifier instead of EOF in export statement"},
+		{"export {yield as", "expected Identifier or String instead of EOF in export statement"},
 		{"export {} from", "expected String instead of EOF in export statement"},
 		{"export {} from", "expected String instead of EOF in export statement"},
 		{"export async", "expected function instead of EOF in export statement"},
@@ -553,7 +562,7 @@ func TestParseError(t *testing.T) {
 		{"function*a(){ (yield=5) => yield }", "unexpected = in expression"},
 		{"function*a(){ (...yield) => yield }", "unexpected yield in arrow function"},
 		{"x = await\n=> a++", "unexpected => in expression"},
-		{"x=async (await,", "unexpected EOF in expression"},
+		{"x=async (await,", "unexpected , in expression"},
 		{"async function a() { class a extends await", "unexpected await in expression"},
 		{"async function a() { await: var a", "unexpected : in expression"},
 		{"async function a() { let await", "unexpected await in binding"},
@@ -567,6 +576,13 @@ func TestParseError(t *testing.T) {
 		{"(async\nfunction(){})", "unexpected function in expression"},
 		{"a + async b", "unexpected b in expression"},
 		{"async await => 5", "unexpected await in arrow function"},
+		{"await: var a", "unexpected : in expression"},
+		{"(await) => 5", "unexpected ) in expression"},
+		{"async\nawait => b", "unexpected => in expression"},
+		{"let\nawait", "unexpected await in binding"},
+		{"let {await = 5} = z", "expected : instead of = in object binding"},
+		{"x = await => a++", "unexpected => in expression"},
+		{"function*a(){ b => yield%5 }", "unexpected % in expression"},
 
 		// specific cases
 		{"{a, if: b, do(){}, ...d}", "unexpected if in expression"}, // block stmt
@@ -583,9 +599,10 @@ func TestParseError(t *testing.T) {
 		{"if (a) 1 else 3", "unexpected else in expression"},
 		{"x = [...]", "unexpected ] in expression"},
 		{"x = {...}", "unexpected } in expression"},
-		{"let\nawait 0", "unexpected 0 in let declaration"},
-		{"const\nawait 0", "unexpected 0 in const declaration"},
-		{"var\nawait 0", "unexpected 0 in var statement"},
+		{"let\nawait 0", "unexpected await in binding"},
+		{"const\nawait 0", "unexpected await in binding"},
+		{"var\nawait 0", "unexpected await in binding"},
+		{"function a(){let\nawait 0", "unexpected 0 in let declaration"},
 
 		// expression to arrow function parameters
 		{"x = ()", "expected => instead of EOF in arrow function"},
@@ -672,7 +689,7 @@ func TestParseError(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.js, func(t *testing.T) {
-			_, err := Parse(parse.NewInputString(tt.js))
+			_, err := Parse(parse.NewInputString(tt.js), Options{})
 			test.That(t, err != io.EOF && err != nil)
 
 			e := err.Error()
@@ -782,6 +799,10 @@ func (sv *ScopeVars) AddExpr(iexpr IExpr) {
 		sv.AddExpr(expr.Y)
 	case *GroupExpr:
 		sv.AddExpr(expr.X)
+	case *CommaExpr:
+		for _, item := range expr.List {
+			sv.AddExpr(item)
+		}
 	}
 }
 
@@ -893,7 +914,7 @@ func TestParseScope(t *testing.T) {
 		{"x={a:b, c=d, ...e};", "", "x=1,b=2,c=3,d=4,e=5"},
 		{"x=[a, b=c, ...d];", "", "x=1,a=2,b=3,c=4,d=5"},
 		{"yield = 5", "", "yield=1"},
-		{"await = 5", "", "await=1"},
+		{"function a(){ await = 5 }", "a=1/", "await=2/await=2"},
 		{"function a(b,c){var d; e = 5; a}", "a=1/b=3,c=4,d=5", "e=2/e=2,a=1"},
 		{"function a(b,c=b){}", "a=1/b=2,c=3", "/"},
 		{"function a(b=c,c){}", "a=1/b=3,c=4", "c=2/c=2"},
@@ -932,9 +953,9 @@ func TestParseScope(t *testing.T) {
 		{"(a=(function b(){})) => a", "/a=1/b=2", "//"},
 		{"label: a", "", "a=1"},
 		{"yield => yield%5", "/yield=1", "/"},
-		{"await => await%5", "/await=1", "/"},
-		{"function*a(){b => yield%5}", "a=1//b=3", "yield=2/yield=2/yield=2"},
-		{"async function a(){b => await%5}", "a=1//b=3", "await=2/await=2/await=2"},
+		{"function a(){ await => await%5 }", "a=1//await=2", "//"},
+		{"function*a(){ b => { yield%5 } }", "a=1//b=3", "yield=2/yield=2/yield=2"},
+		{"async function a(){ b => await%5 }", "a=1//b=3", "await=2/await=2/await=2"},
 		{"let a; {let b = a}", "a=1/b=2", "/a=1"},
 		{"let a; {var b}", "a=1,b=2/", "/b=2"}, // may remove b from uses
 		{"let a; {var b = a}", "a=1,b=2/", "/b=2,a=1"},
@@ -983,10 +1004,11 @@ func TestParseScope(t *testing.T) {
 		{`for(let b of c){let b;{b}}`, "/b=2,b=3/", "c=1/c=1/b=3"},
 		{`for(var b of c){let b;{b}}`, "b=1/b=3/", "c=2/b=1,c=2/b=3"},
 		{`for(var b of c){var b;{b}}`, "b=1//", "c=2/b=1,c=2/b=1"},
+		{`function a(b){for(let c of b){let b;}}`, "a=1/b=2/c=3,b=4", "//b=2"},
 	}
 	for _, tt := range tests {
 		t.Run(tt.js, func(t *testing.T) {
-			ast, err := Parse(parse.NewInputString(tt.js))
+			ast, err := Parse(parse.NewInputString(tt.js), Options{})
 			if err != io.EOF {
 				test.Error(t, err)
 			}
@@ -1003,7 +1025,7 @@ func TestParseScope(t *testing.T) {
 
 func TestScope(t *testing.T) {
 	js := "let a,b; b = 5; var c; {d}{{d}}"
-	ast, err := Parse(parse.NewInputString(js))
+	ast, err := Parse(parse.NewInputString(js), Options{})
 	if err != io.EOF {
 		test.Error(t, err)
 	}
@@ -1023,9 +1045,9 @@ func TestScope(t *testing.T) {
 }
 
 func TestParseInputError(t *testing.T) {
-	_, err := Parse(parse.NewInput(test.NewErrorReader(0)))
+	_, err := Parse(parse.NewInput(test.NewErrorReader(0)), Options{})
 	test.T(t, err, test.ErrPlain)
 
-	_, err = Parse(parse.NewInput(test.NewErrorReader(1)))
+	_, err = Parse(parse.NewInput(test.NewErrorReader(1)), Options{})
 	test.T(t, err, test.ErrPlain)
 }
